"""Class to load pre-built data set information from JSON files in source .  All results are cached"""
import os.path as osp
import json
from collections import defaultdict, namedtuple
from lrgasp import LrgaspException
from lrgasp.objDict import ObjDict
from lrgasp.defs import Species, Sample, LibraryPrep, Platform

##
# LRGASP run metadata files in lib/lrgasp/data/
##
lrgasp_run_metadata_files = ("encode-metadata.json", "simulated-metadata.json", )


class LrgaspRun(ObjDict):
    """Used to create an LRGASP run object for sequencing data. Class is only used
    to serialized, ObjDict is used to access when deserialized."""
    def __init__(self, *, species, sample, run_acc, description, library_prep, platform):
        self.species = species
        self.sample = sample
        self.run_acc = run_acc
        self.description = description
        self.library_prep = library_prep
        self.platform = platform
        self.replicates = None

class LrgaspReplicate(ObjDict):
    """Collection of files for a replicate within a run.   Note that a given
    Biosample and replicated can be used by multiple runs.  Class is only used
    to serialized, ObjDict is used to access when deserialized."""
    def __init__(self, replicate_number, biosample_accs, size_range):
        self.replicate_number = replicate_number
        self.biosample_accs = biosample_accs  # deserialized as a sorted tuple
        self.size_range = size_range
        self.files = []
        # self.run_md = added when deserialized

class LrgaspRnaSeqFile(ObjDict):
    """One data file from an RNA-Seq run, used to build JSON in a consistent way
    Class is only used to serialized, ObjDict is used to access when deserialized.
    When deserialized paired files will be link
    """
    def __init__(self, *, file_acc, file_type, url, s3_uri, file_size, md5sum, run_acc, biological_replicate_number,
                 output_type, paired_end=None, paired_with=None):
        self.file_acc = file_acc
        self.file_type = file_type
        self.url = url
        self.s3_uri = s3_uri
        self.file_size = file_size
        self.md5sum = md5sum
        self.run_acc = run_acc
        self.biological_replicate_number = biological_replicate_number
        self.output_type = output_type
        self.paired_end = paired_end

        self.paired_with = paired_with
        # self.paired_file is built when deserialized
        # self.replicate_md = added when deserialized

class RunType(namedtuple("RunType",
                         ("sample", "library_prep", "platform"))):
    "attributes describing a run"

    def __str__(self):
        return str(self.sample) + '/' + str(self.library_prep) + '/' + str(self.platform)

class LrgaspRnaSeqMetaData(list):
    """deserialized LRSGAP RNA-Seq metadata, along with access methods"""
    cache = None

    def __init__(self):
        self.file_md_by_acc = {}
        self.run_by_acc = {}

        # indexed by (library_prep, platform)
        self.runs_by_prep_platform = defaultdict(list)

        # this is indexed by both individual biosample accs and by sorted
        # tuple of accs to handle mixes.  The same values can map to replicates
        # in different sequences technologies, hence it is a list
        self.replicate_by_biosample_acc = defaultdict(list)

    def finish(self):
        self.runs_by_prep_platform.default_factory = None
        self.replicate_by_biosample_acc.default_factory = None

    def _edit_run_types(self, run_md):
        "conversion to SymEnum"
        run_md.species = Species(run_md.species)
        run_md.sample = Sample(run_md.sample)
        run_md.library_prep = LibraryPrep(run_md.library_prep)
        run_md.platform = Platform(run_md.platform)

    def _add_file(self, replicate_md, file_md):
        if file_md.file_acc in self.file_md_by_acc:
            raise LrgaspException(f"duplicate file accession '{file_md.file_acc}'")
        self.file_md_by_acc[file_md.file_acc] = file_md
        file_md.replicate_md = replicate_md
        if file_md.paired_file is not None:
            self.file_md_by_acc[file_md.paired_file.file_acc] = file_md.paired_file

    def _add_files(self, replicate_md, file_mds):
        for file_md in file_mds:
            self._add_file(replicate_md, file_md)

    def _add_biosample(self, replicate_md):
        # add by both all samples and sorted tuple
        self.replicate_by_biosample_acc[replicate_md.biosample_accs] = replicate_md
        for acc in replicate_md.biosample_accs:
            self.replicate_by_biosample_acc[acc] = replicate_md

    def _add_replicate(self, run_md, replicate_md):
        replicate_md.run_md = run_md
        self._add_biosample(replicate_md)
        self._add_files(replicate_md, replicate_md.files)

    def add(self, run_md):
        self._edit_run_types(run_md)
        if run_md.run_acc in self.run_by_acc:
            raise LrgaspException(f"duplicate run id '{run_md.run_acc}'")
        self.append(run_md)
        self.run_by_acc[run_md.run_acc] = run_md
        self.runs_by_prep_platform[run_md.library_prep, run_md.platform].append(run_md)
        for replicate_md in run_md.replicates:
            self._add_replicate(run_md, replicate_md)

    def get_file_by_acc(self, file_acc):
        try:
            return self.file_md_by_acc[file_acc]
        except KeyError:
            raise LrgaspException(f"unknown LRGASP file accession '{file_acc}', file accession must be one in the LRGASP RNA-Seq data matrix")

    def get_run_by_acc(self, run_acc):
        try:
            return self.run_by_acc[run_acc]
        except KeyError:
            raise LrgaspException(f"unknown LRGASP run accession '{run_acc}', run accession should be one in the LRGASP RNA-Seq data matrix")

    def get_run_by_file_acc(self, file_acc):
        fil = self.get_file_by_acc(file_acc)
        return self.get_run_by_acc(fil.run_acc)

    def get_runs_by_prep_platform(self, library_prep, platform):
        "get runs for prep/platform"
        return self.runs_by_prep_platform.get((library_prep, platform), ())

def get_run_type(run_md):
    return RunType(run_md.sample, run_md.library_prep, run_md.platform)

def _pair_files(file_mds):
    "linked paired ends files and construct list of pairs"
    files_by_acc = {f.file_acc: f for f in file_mds}
    paired_files = []
    while len(files_by_acc) > 0:
        _, file_md = files_by_acc.popitem()
        if file_md.paired_end is None:
            file_md.paired_file = None
            paired_files.append(file_md)
        else:
            # put in right order
            paired_md = files_by_acc.pop(file_md.paired_with)
            p1, p2 = (file_md, paired_md) if file_md.paired_end < paired_md.paired_end else (paired_md, file_md)
            p1.paired_file = p2
            p2.paired_file = p1
            paired_files.append(p1)
    return paired_files

def _edit_run(run_md):
    """Modify deserialized run. Link paired end files and convert biosample_accs
    to a sorted tuple"""
    for rep in run_md.replicates:
        rep.files = _pair_files(rep.files)
        rep.biosample_accs = tuple(sorted(rep.biosample_accs))
    return run_md

def _load_lrgasp_rna_seq_metadata_file(rna_seq_md, metadata_json):
    with open(metadata_json) as fh:
        for run_md in json.load(fh, object_pairs_hook=ObjDict):
            rna_seq_md.add(_edit_run(run_md))

def _load_lrgasp_rna_seq_metadata_files():
    "load of all metadata files when not cached"
    rna_seq_md = LrgaspRnaSeqMetaData()
    for json_file in lrgasp_run_metadata_files:
        json_path = osp.join(osp.dirname(__file__), "data", json_file)
        try:
            _load_lrgasp_rna_seq_metadata_file(rna_seq_md, json_path)
        except Exception as ex:
            raise LrgaspException(f"failed to load '{json_path}'") from ex
    rna_seq_md.finish()
    return rna_seq_md

def get_lrgasp_rna_seq_metadata():
    """get LRGASP metadata, possible cached"""
    if LrgaspRnaSeqMetaData.cache is None:
        LrgaspRnaSeqMetaData.cache = _load_lrgasp_rna_seq_metadata_files()
    return LrgaspRnaSeqMetaData.cache



"""
Operations associated with logging
"""
import logging
import os
import sys
from logging.handlers import SysLogHandler

# setup from command line
_lrgasp_logger = None

def getLrgaspLogger():
    return _lrgasp_logger

def getFacilityNames():
    return tuple(SysLogHandler.facility_names.keys())


def getLevelNames():
    return tuple(sorted(logging._nameToLevel.keys()))


def parseFacility(facilityStr):
    """Convert case-insensitive facility string to a facility number."""
    facility = SysLogHandler.facility_names.get(facilityStr.lower())
    if facility is None:
        raise ValueError("invalid syslog facility: \"{}\"".format(facilityStr))
    return facility


def parseLevel(levelStr):
    "convert a log level string to numeric value"
    level = logging._nameToLevel.get(levelStr.upper())
    if level is None:
        raise ValueError("invalid logging level: \"{}\"".format(levelStr))
    return level


def _convertFacility(facility):
    """convert facility from string to number, if not already a number"""
    return facility if isinstance(facility, int) else parseFacility(facility)


def _convertLevel(level):
    """convert level from string to number, if not already a number"""
    return level if isinstance(level, int) else parseLevel(level)

def _loggerBySpec(logger):
    """Logger maybe a logger, logger name, or None for default logger, returns the
    logger."""
    if not isinstance(logger, logging.Logger):
        logger = logging.getLogger(logger)
    return logger

def setupLogger(logger, handler, formatter=None, level=None):
    """add handle to logger and set logger level to the minimum of it's current
    and the handler level.  Logger maybe a logger, logger name, or None for
    default logger, returns the logger.

    """
    logger = _loggerBySpec(logger)
    if level is not None:
        logger.setLevel(_convertLevel(level))
    if handler.level is not None:
        logger.setLevel(min(handler.level, logger.level))
    logger.addHandler(handler)
    if formatter is not None:
        handler.setFormatter(formatter)
    return logger

def setupStreamLogger(logger, fh, level, formatter=None):
    """Configure logging to a specified open file.  Logger maybe a logger or
    logger name, returns the logger."""
    level = _convertLevel(level)
    handler = logging.StreamHandler(stream=fh)
    handler.setLevel(level)
    return setupLogger(logger, handler, formatter, level=level)


def setupStderrLogger(logger, level, formatter=None):
    """configure logging to stderr.  Logger maybe a logger, logger name, or
    None for default logger, returns the logger."""
    return setupStreamLogger(logger, sys.stderr, _convertLevel(level), formatter)


def getSyslogAddress():
    """find the address to use for syslog"""
    for dev in ("/dev/log", "/var/run/syslog"):
        if os.path.exists(dev):
            return dev
    return ("localhost", 514)


def setupSyslogLogger(logger, facility, level, prog=None, address=None, formatter=None):
    """configure logging to syslog based on the specified facility.  If prog
    specified, each line is prefixed with the name.  Logger maybe a logger or
    logger name, returns the logger."""
    if address is None:
        address = getSyslogAddress()
    handler = SysLogHandler(address=address, facility=facility)
    # add a formatter that includes the program name as the syslog ident
    if prog is not None:
        handler.setFormatter(logging.Formatter(fmt="{} %(message)s".format(prog)))
    handler.setLevel(level)
    return setupLogger(logger, handler, formatter)


def setupNullLogger(logger, level=None):
    "configure discard logging.  Returns logger."
    handler = logging.NullHandler()
    if level is not None:
        handler.setLevel(_convertLevel(level))
    return setupLogger(logger, handler)


def addCmdOptions(parser, *, defaultLevel=logging.WARN):
    """
    Add command line options related to logging.  None of these are defaulted,
    as one might need to determine if they were explicitly set. The use case
    being getting a value from a configuration file if it is not specified on
    the command line.
    """
    # want to validate name, but want to store the string in the arguments vector
    # rather than the numeric value.
    def validateFacility(facilityStr):
        parseFacility(facilityStr)
        return facilityStr

    def validateLevel(levelStr):
        parseLevel(levelStr)
        return levelStr

    parser.add_argument("--syslogFacility", type=validateFacility,
                        help="Set syslog facility to case-insensitive symbolic value, if not specified, logging is not done to stderr, "
                        " one of {}".format(", ".join(getFacilityNames())))
    parser.add_argument("--logStderr", action="store_true",
                        help="also log to stderr, even when logging to syslog")
    parser.add_argument("--logLevel", type=validateLevel, default=defaultLevel,
                        help="Set level to case-insensitive symbolic value, one of {}".format(", ".join(getLevelNames())))
    parser.add_argument("--logConfFile",
                        help="Python logging configuration file, see logging.config.fileConfig()")
    parser.add_argument("--logDebug", action="store_true",
                        help="short-cut that that sets --logStderr and --logLevel=DEBUG")


def setupFromCmd(opts, *, logger=None, prog=None):
    """configure logging based on command options. Prog is used it to set the
    syslog program name. If prog is not specified, it is obtained from sys.arg.
    Logger maybe a logger, logger name, or None for default logger, returns the logger.

    N.B: logging must be initialized after daemonization
    """
    if opts.logDebug:
        opts.logStderr = True
        opts.logLevel = logging.DEBUG
    if prog is None:
        prog = os.path.basename(sys.argv[0])
    logger = _loggerBySpec(logger)
    level = _convertLevel(opts.logLevel) if opts.logLevel is not None else logging.WARN
    if opts.syslogFacility is not None:
        setupSyslogLogger(logger, opts.syslogFacility, level, prog=prog)
    if (opts.syslogFacility is None) or opts.logStderr:
        setupStderrLogger(logger, level)
    if opts.logConfFile is not None:
        logging.config.fileConfig(opts.logConfFile)
    global _lrgasp_logger
    _lrgasp_logger = logger
    return logger


class StreamToLogger(object):
    """
    File-like stream object that redirects writes to a logger instance.
    """
    def __init__(self, logger, level):
        self.logger = logger
        self.level = level
        self.linebuf = ''

    def write(self, buf):
        for line in buf.rstrip().splitlines():
            self.logger.log(self.level, line.rstrip())

    def flush(self):
        pass



"""various functions and class for access to synpase and comparing files with local files"""

import sys
import os.path as osp
import json
from lrgasp.loggingOps import getLrgaspLogger
from lrgasp import LrgaspException


# synpase upload has an really annoying spinner, disable it
import synapseclient.core.utils as synapseutils

def noop_print_tick(self):
    pass

synapseutils.Spinner.print_tick = noop_print_tick

import synapseclient
import synapseutils
from synapseclient import Synapse
from synapseclient.__main__ import login_with_prompt  # function used by synapse command
from lrgasp.objDict import ObjDict
from lrgasp import prog_error_excepts
from lrgasp.defs import Challenge

prog_error_syn_excepts = (synapseclient.core.exceptions.SynapseError, ) + prog_error_excepts

class SynNode:
    "Node in the synapse entity hierarchy."
    def __init__(self, entity):
        self.entity = entity
        self.children = []

class SynTree:
    """Tree of synapse entities (Project, Folder, File, Link)."""
    def __init__(self):
        self.root = None
        self.node_by_synid = {}

    def add(self, entity):
        node = SynNode(entity)
        self.node_by_synid[node.entity.id] = node
        if self.root is None:
            self.root = node
        else:
            parent = self.node_by_synid[entity.parentId]
            parent.children.append(node)
        return node

def syn_tree_load(syn, root_synid):
    """Load existing file hierarchy information from synapse into a SynNode tree"""
    syn_tree = SynTree()
    for root, dir, files in synapseutils.walk(syn, root_synid):
        syn_tree.add(syn.get(dir))
        for f in files:
            syn_tree.add(syn.get(f))
    return syn_tree

class FileNode:
    """Node in the file hierarchy, for uploading or comparing against
    synapse"""
    def __init__(self, filename, parent, isdir):
        self.filename = filename
        self.isdir = isdir
        self.parent = parent
        self.children = []

    def get_path(self, root_dir=None):
        path = []
        n = self
        while n is not None:
            path.insert(0, n.filename)
            n = n.parent
        if root_dir is not None:
            path.insert(0, root_dir)
        return osp.join(*path)

    def dump(self, indent, fh=sys.stderr):
        print((4 * indent * ' ') + self.filename + ('/' if self.isdir > 0 else ''),
              file=fh)
        for node in self.children:
            node.dump(indent + 1, fh)

class FileTree:
    """Tree of files and directories, build from metadata"""
    def __init__(self):
        self.root = None

    def add(self, filepath, parent, isdir=False):
        filename = osp.basename(filepath)
        node = FileNode(filename, parent, isdir)
        if self.root is None:
            assert parent is None
            self.root = node
        else:
            parent.children.append(node)
        return node

    def dump(self, fh=sys.stderr):
        print("FileTree", file=fh)
        if self.root is not None:
            self.root.dump(1, fh)

def add_login_args(parser):
    "add options for logging into synapse"
    parser.add_argument("--test_config_json",
                        help="JSON configuration file for doing testing.  See  See lrgasp.synapse_access.LrgaspSynapseConfig for information.")
    parser.add_argument('-u', '--username', dest='synapseUser',
                        help='Username or email address used to connect to Synapse; if not specified must already be logged in with the --rememberMe option to this program or the synapse program')
    parser.add_argument('-p', '--password', dest='synapsePassword',
                        help='Password or api key used to connect to Synapse')
    parser.add_argument('--rememberMe', '--remember-me', dest='rememberMe', action='store_true', default=False,
                        help='Cache credentials for automatic authentication on future interactions with Synapse')

def syn_connect(args):
    "connect and log in based on options"
    syn_conf = LrgaspSynConfig.factory(args)
    syn = Synapse()
    syn.logger.setLevel(getLrgaspLogger().level)  # cut down on noise

    # None user/password uses cache or prompts.  Command line overrides conf
    user = args.synapseUser if args.synapseUser is not None else syn_conf.user
    password = args.synapsePassword if args.synapsePassword is not None else syn_conf.password
    login_with_prompt(syn, user, password, rememberMe=args.rememberMe)
    getLrgaspLogger().debug(f"logged in as synpase user '{syn.username}'")
    return syn

def get_project_by_name(syn, project_name):
    synid = syn.findEntityId(project_name)
    if synid is None:
        raise LrgaspException(f"synapse project not found '{project_name}'")
    return synid

class LrgaspSynConfig:
    """Object contain configuration for accessing synapse.  For production
    submission, this has the predefined ids from production_lrgasp_syn_ids
    and uses the login prompted or from the --rememberMe cached.

    For testing, the fields are loaded from a JSON file and include a test
    user and password/app key.

    The test JSON file should have the fields 'user' and 'password', and
    fields from set in _init_production
    """
    @classmethod
    def _production_conf(cls):
        self = cls()
        self.user = None
        self.password = None
        self.lrgasp_synchallenge_name = 'LRGASP'
        self.lrgasp_synchallenge_id = 4441
        self.participants_team_name = 'LRGASP Participants',
        self.syn_queue_ids = {
            Challenge.iso_detect_ref: 9614748,
            Challenge.iso_quant: 9614749,
            Challenge.iso_detect_de_novo: 9614750,
        }
        return self

    @classmethod
    def _test_conf(cls, test_config_json):
        with open(test_config_json) as fh:
            conf = json.load(fh, object_pairs_hook=ObjDict)
        self = cls()
        self.user = conf.user
        self.password = conf.password
        self.lrgasp_synchallenge_name = conf.lrgasp_synchallenge_name
        self.lrgasp_synchallenge_id = conf.lrgasp_synchallenge_id
        self.participants_team_name = conf.participants_team_name
        self.syn_queue_ids = {}
        for key, val in conf.syn_queue_ids.items():
            self.syn_queue_ids[Challenge(key)] = val
        return self

    @classmethod
    def factory(cls, args):
        if args.test_config_json is not None:
            return cls._test_conf(args.test_config_json)
        else:
            return cls._production_conf()



"""
Load and validate model and expression entries.  This does full validation of
all metadata and data files.
"""
from lrgasp import LrgaspException
from lrgasp.defs import Challenge
from lrgasp import entry_metadata
from lrgasp import model_data
from lrgasp import de_novo_rna_data
from lrgasp import read_model_map_data
from lrgasp import expression_data
from lrgasp.experiment_metadata import get_models_gtf, get_read_model_map_tsv, get_rna_fasta, get_expression_tsv

def _model_map_transcript_ids(read_model_map):
    "get list of all read_model_map transcript ids"
    return sorted(set([p.transcript_id for p in read_model_map if p.transcript_id is not None]))

def _validate_model_to_read_mapping(transcript_id, read_model_map):
    if read_model_map.get_by_transcript_id(transcript_id) is None:
        raise LrgaspException(f"transcript in models '{transcript_id}' not in read-model_map")

def _validate_read_mapping_to_model(transcript_id, models):
    # transcript might be None if specified as `*'
    if (transcript_id is not None) and (models.by_transcript_id.get(transcript_id) is None):
        raise LrgaspException(f"transcript in read_model_map '{transcript_id}' not in models")

def validate_ref_model_and_read_mapping(models, read_model_map):
    # all model mapping must be in models
    for transcript_id in _model_map_transcript_ids(read_model_map):
        _validate_read_mapping_to_model(transcript_id, models)
    # all transcripts must be in model map
    for trans in models:
        _validate_model_to_read_mapping(trans.transcript_id, read_model_map)

def _validate_ref_model_experiment(experiment_md):
    models_gtf = get_models_gtf(experiment_md)
    map_file = get_read_model_map_tsv(experiment_md)
    try:
        models = model_data.load(models_gtf)
        read_model_map = read_model_map_data.load(map_file)
        validate_ref_model_and_read_mapping(models, read_model_map)
    except Exception as ex:
        raise LrgaspException(f"validation failed on '{models_gtf}' and '{map_file}'") from ex

def _validate_de_novo_rna_to_read_mapping(transcript_id, read_model_map):
    if read_model_map.get_by_transcript_id(transcript_id) is None:
        raise LrgaspException(f"transcript in de novo RNAs '{transcript_id}' not in read-model_map")

def _validate_read_mapping_to_de_novo_rna(transcript_id, de_novo_rna_ids):
    # transcript might be None if specified as `*'
    if (transcript_id is not None) and (transcript_id not in de_novo_rna_ids):
        raise LrgaspException(f"transcript in read_model_map '{transcript_id}' not in de novo RNAs")

def validate_de_novo_rna_and_read_mapping(de_novo_rna_ids, read_model_map):
    # all model mapping must be in reads
    for transcript_id in _model_map_transcript_ids(read_model_map):
        _validate_read_mapping_to_de_novo_rna(transcript_id, de_novo_rna_ids)
    # all transcripts must be in model map
    for transcript_id in de_novo_rna_ids:
        _validate_de_novo_rna_to_read_mapping(transcript_id, read_model_map)

def _validate_de_novo_model_experiment(experiment_md):
    rna_fasta = get_rna_fasta(experiment_md)
    map_file = get_read_model_map_tsv(experiment_md)
    try:
        de_novo_rna_ids = de_novo_rna_data.load(rna_fasta)
        read_model_map = read_model_map_data.load(map_file)
        validate_de_novo_rna_and_read_mapping(de_novo_rna_ids, read_model_map)
    except Exception as ex:
        raise LrgaspException(f"validation failed on '{rna_fasta}' with '{map_file}'") from ex

def validate_expression_and_model(models, expression_mat):
    # all expression matrix ids must be in models
    for row in expression_mat.df.iterrows():
        if row[1].ID not in models.by_transcript_id:
            raise LrgaspException(f"expression matrix ID '{row[1].ID}' not found in models")

def _validate_expression_experiment(experiment_md):
    models_gtf = get_models_gtf(experiment_md)
    expression_tsv = get_expression_tsv(experiment_md)
    try:
        models = model_data.load(models_gtf)
        expression_mat = expression_data.load(expression_tsv, experiment_md)
        validate_expression_and_model(models, expression_mat)
    except Exception as ex:
        raise LrgaspException(f"validation failed on '{models_gtf}' with '{expression_tsv}'") from ex

def validate_experiment_data(entry_md, experiment_md):
    try:
        if experiment_md.challenge_id == Challenge.iso_detect_ref:
            _validate_ref_model_experiment(experiment_md)
        elif experiment_md.challenge_id == Challenge.iso_detect_de_novo:
            _validate_de_novo_model_experiment(experiment_md)
        elif experiment_md.challenge_id == Challenge.iso_quant:
            _validate_expression_experiment(experiment_md)
        else:
            raise LrgaspException("bug")
    except Exception as ex:
        raise LrgaspException(f"validation of experiment '{experiment_md.experiment_id}' failed: {experiment_md.experiment_json}") from ex

def _entry_data_validate(entry_md):
    entry_metadata.load_experiments_metadata(entry_md)
    for experiment_md in entry_md.experiments:
        validate_experiment_data(entry_md, experiment_md)

def entry_data_validate(entry_dir):
    """load and validate all metadata and data files for an entry, ensuring
    consistency."""
    entry_md = entry_metadata.load_dir(entry_dir)
    try:
        _entry_data_validate(entry_md)
    except Exception as ex:
        raise LrgaspException(f"validation of entry '{entry_md.entry_id}' failed: {entry_md.entry_json}") from ex

def entry_metadata_validate(entry_dir):
    """load and validate all metadata for an entry. This does not check data."""
    entry_md = entry_metadata.load_dir(entry_dir)
    try:
        entry_metadata.load_experiments_metadata(entry_md)
    except Exception as ex:
        raise LrgaspException(f"validation of entry metadata '{entry_md.entry_id}' failed: {entry_md.entry_json}") from ex
